@@ -20,6 +20,7 @@ Command-line interface to the OpenStack Nova API.
 
 import argparse
 import logging
+import os
 import sys
 
 from keystoneauth1 import loading
@@ -365,6 +366,9 @@ class OpenStackComputeShell(object):
             help=_("Use this API endpoint instead of the Service Catalog. "
                    "Defaults to env[OS_ENDPOINT_OVERRIDE]."))
 
+        parser.set_defaults(func=self.do_help)
+        parser.set_defaults(command='')
+
         if osprofiler_profiler:
             parser.add_argument('--profile',
                                 metavar='HMAC_KEY',
@@ -813,9 +817,19 @@ class OpenStackHelpFormatter(argparse.HelpFormatter):
         super(OpenStackHelpFormatter, self).start_section(heading)
 
 
-def main():
+def main(argv=sys.argv[1:]):
     try:
-        argv = [encodeutils.safe_decode(a) for a in sys.argv[1:]]
+        # Special dansmith envvar to hide the warning. Don't rely on this
+        # because we will eventually remove all this stuff.
+        if os.environ.get("NOVACLIENT_ISHOULDNTBEDOINGTHIS") != "1":
+            print(
+                _(
+                    "nova CLI is deprecated and will be removed in a future "
+                    "release"
+                ),
+                file=sys.stderr,
+            )
+        argv = [encodeutils.safe_decode(a) for a in argv]
         OpenStackComputeShell().main(argv)
     except Exception as exc:
         logger.debug(exc, exc_info=1)
